#ifndef CLIENT_H
#define CLIENT_H

#include <QTcpSocket>
#include <QImage>
#include "imageprovider.h"
#include "config.h"

QT_BEGIN_NAMESPACE
class QTcpSocket;
class QNetworkSession;
class QHostAddress;
class QSettings;
QT_END_NAMESPACE

class Client : public Config
{
    Q_OBJECT

public:
    Client(QObject *parent = 0);
    QImage clientImage;
    ImageProvider imageProvider;
//    Config config;

    Q_INVOKABLE QImage getImage() const {
        return clientImage;
    }

signals:
    void imageAvailable();

public slots:
    void sendString(const QString &msg);
    void interpretCommandAndSendMessage(const QString &msg);
    void sendMessage(const QString &cmd, const QStringList &msgList);
    void startTransfer();
    void setRootObject(QObject *object);
    void addPatient(QString idNumber, QString title, QString firstName, QString lastName);
    void addDoctor(QString idNumber, QString title, QString firstName, QString lastName);
    void clearPatients();
    void clearDoctors();

private slots:
    void readNextItem();
    void readString();
    void readStringList();
    void readImage();
    void displayError(QAbstractSocket::SocketError socketError);
    void sessionOpened();


private:
    void interpretAnswer(const QString uniqueId, const QString &cmd, const QStringList &msgList);
    QTcpSocket *tcpSocket;
    quint16 blockSize;
    QByteArray block;
    bool nextConnexionIsAnImage;

    QNetworkSession *networkSession;
    QObject *rootObject;
};

#endif // CLIENT_H
