/*! \brief DrCute is the client part of the DrCute/MrHyde software ensemble.
 *
 *   The client (Dr.Cute) makes use of the network to communicate with the server.
 *   The C++ code of the client is just there to establish the link between the QML user interface and the server.
 */

#include <QtGui/QApplication>
#include <QtDeclarative>
#include "client.h"


int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    // Those definitions are required to store and retrieve the application settings
    QCoreApplication::setOrganizationName("0x4e84");
    QCoreApplication::setApplicationName("DrCute");

    // The objects we need for the program
    Client client;
    QDeclarativeView view;

    // Linking to the QML main file
    view.rootContext()->setContextProperty("client", &client);
    //view.rootContext()->setContextProperty("clientImage", client.clientImage);

    view.engine()->addImageProvider("imageprovider", &client.imageProvider);

    view.setSource(QUrl("qrc:/qml/drcute/main.qml"));
    view.setResizeMode(QDeclarativeView::SizeRootObjectToView);
    QObject *mainQml = view.rootObject();
    client.setRootObject(mainQml);

    QObject *object = view.rootObject();
    object->setProperty("width", 800);
    object->setProperty("ipaddress", client.getIpAddress());
    object->setProperty("port", client.getPort());

    // Connecting the signals and slots between the QML main page and the "client" object
    QObject::connect(mainQml, SIGNAL(send(QString)),
                     &client, SLOT(interpretCommandAndSendMessage(QString)));
    QObject::connect(mainQml, SIGNAL(sendIP(QString)),
                     &client, SLOT(setIpAddress(QString)));
    QObject::connect(mainQml, SIGNAL(sendPort(QString)),
                     &client, SLOT(setPort(QString)));
    QObject::connect(mainQml, SIGNAL(quit()),
                     &app,    SLOT(quit()));
    QObject::connect(mainQml, SIGNAL(showNormal()),
                     &view,   SLOT(showNormal()));

    // ...and finally showing it!
    view.showFullScreen();
//    view.show();

    return app.exec();
}
