import QtQuick 1.0

Rectangle {
    id: list

    // Signals
    signal close
    signal select

    property int space: globalRadius + globalSpacing
    property int fontsize: smallFontSize
    property string title
    property string iconName
    property string selectedId
    property string selectedTitle
    property string selectedFirstName
    property string selectedLastName

    anchors.horizontalCenter: parent.horizontalCenter
    anchors.verticalCenter: parent.verticalCenter

    width: 100
    height: 100

    color: backgroundColor
    radius: globalRadius
    border.color: borderColor
    border.width: borderWidth

    state: "visible"
    visible: opacity > 0

    Image {
        id: icon
        source: "qrc:/images/" + iconName + ".png"
        anchors.horizontalCenter: parent.horizontalCenter
        y: - icon.height
    }

    ListModel {
        id: listModel
        // This model is populated later on using the 'addItem' function
    }

    // The delegate for the items
    Component {
        id: itemDelegate
        Rectangle {
            id: itemFrame
            height: childrenRect.height
            width: listViewFrame.width - 2*globalRadius
            anchors.leftMargin: globalRadius
            anchors.rightMargin: globalRadius
            property bool isSelected: false
            property string nonHoveredColor: backgroundColor
            color: isSelected ? selectedItemColor : nonHoveredColor

            Row {
                id: itemRow
                anchors.horizontalCenter: itemFrame.horizontalCenter
                spacing: globalRadius*2
                Text { text: myIdNumber;  color: textColor; font.pointSize: fontsize }
                Text { text: myTitle;     color: textColor; font.pointSize: fontsize }
                Text { text: myFirstName; color: textColor; font.pointSize: fontsize }
                Text { text: myLastName;  color: textColor; font.pointSize: fontsize }
            }

            MouseArea {
                id: itemMouseArea
                anchors.fill: itemFrame
                hoverEnabled: true
                onClicked: {
                    selectedId        = myIdNumber;
                    selectedTitle     = myTitle
                    selectedFirstName = myFirstName
                    selectedLastName  = myLastName
                    itemFrame.focus= true
                }
            }

            // WARNING: This causes the selection to be lost when it is no more visible...
            onFocusChanged: {
                itemFrame.isSelected = itemFrame.focus;
            }

            states:
                State {
                    name: "hovered"
                    when: itemMouseArea.containsMouse
                    PropertyChanges { target: itemFrame; nonHoveredColor: hoveredItemColor }
                }

            transitions: Transition {
                from: "hovered"
                ColorAnimation { duration: 100 }
            }
        }

    }

    Column {
        id: columnContainer
        spacing: globalRadius

        anchors.horizontalCenter: list.horizontalCenter

        // Used to force columnContainer to add an additional spacing on top...
        Item {
            height: 1
            width: 1
        }

        Text {
            id: titleText
            anchors.horizontalCenter: columnContainer.horizontalCenter
            font.pointSize: largeFontSize
            color: textColor
            text: title
        }

        Rectangle {
            id: listViewFrame
            anchors.horizontalCenter: columnContainer.horizontalCenter
            width: list.width - 2*globalRadius
            height: list.height - closeButton.height - titleText.height - 4*globalRadius

            color: backgroundColor
            border.color: borderColor
            border.width: borderWidth

            ListView {
                id: listView
                spacing: globalSpacing
                clip: true
                anchors.fill: listViewFrame
                anchors.topMargin: globalRadius
                anchors.bottomMargin: globalRadius
                anchors.leftMargin: globalRadius
                anchors.rightMargin: globalRadius

                model: listModel
                delegate: itemDelegate

                focus: true
            }
        }

        Row {
            id: buttonRow
            anchors.horizontalCenter: columnContainer.horizontalCenter
            spacing: globalRadius

            CuteButton {
                id: selectButton
                label: "Select"
                anchors.verticalCenter: buttonRow.verticalCenter
                onClicked: {
                    list.select()
                }
            }

            CuteButton {
                id: closeButton
                label: "Close"
                anchors.verticalCenter: buttonRow.verticalCenter
                onClicked: {
                    list.close()
                }
            }
        }
    }

    states: [
        State {
            name: "invisible"
            PropertyChanges { target: list; opacity: 0 }
            PropertyChanges { target: parent; focus: true }
        },

        State {
            name: "visible"
            PropertyChanges { target: list; opacity: 1.0 }
        }
    ]

    transitions: Transition {
        NumberAnimation { properties: "opacity"; duration: 200 }
    }

    function toggle() {
        if (state == "visible")
            state = "invisible";
        else
            state = "visible";
    }

    function addItem(idNumber, title, firstName, lastName) {
        listModel.append( {myIdNumber: idNumber, myTitle: title, myFirstName: firstName, myLastName: lastName} )
    }

    function clearAll() {
        listModel.clear()
    }

}
