import QtQuick 1.0

Item {
    id: menu
    signal loadCompleted

    property string menuLabel
    property string hPosition: "Left"
    property string vPosition: "Top"

    property bool headerLarge: false
    property bool headerVisible: true
    property bool itemsVisible: false
    property bool folded: false

    property string header
    property string header2
    property string header3
    property string item1
    property string item2
    property string item3
    property string item4
    property int itemCount: ((item1=="") ? 0:1) + ((item2=="") ? 0:1) +((item3=="") ? 0:1) + ((item4=="") ? 0:1)

    width: menuItemWidth
    height: menuColumn.visible ? (smalliconsize + globalSpacing) * itemCount + largeiconsize + globalRadius * 2 :
                                 headerItem.height + globalSpacing * 2

    x: (hPosition == "Right")  ? (parent.width  - width - globalSpacing) : globalSpacing
    y: (vPosition == "Bottom") ? (parent.height - height - globalSpacing) : globalSpacing

    state: "headerIconLabelSmall"

    function toggleState() {
        if (state == "open") state = "headerIconLabelSmall";
        else if ((state == "headerIconSmall") ||
                 (state == "headerIconLarge") ||
                 (state == "headerIconLabelSmall") ||
                 (state == "headerIconLabelLarge")) state = "open";
    }

    Rectangle {
        id: menuBkgd
        color: backgroundColor
        height: menu.height + 1
        width: menu.width + globalRadius *2
        y: 0 //globalRadius
        x: - globalRadius + 1
    }

    // Menu header
    Menuitem {
        id: headerItem
        y: (vPosition == "Bottom") ? menu.height - headerItem.height : 0
        label: header
        label2: header2
        label3: header3
        name: menuLabel
        property bool large: true
        state: "open"
        iconsize: large ? largeiconsize : smalliconsize
        isHeader: true
        menuFolded: true
        hPosition: menu.hPosition
        vPosition: menu.vPosition
        onHeaderClicked: menu.toggleState()
        onItemClicked: menu.state = "headerIconLabelSmall"
    }


    // Implementing menu
    ListModel {
        id: menuModel

        // ListModel does not allow to define elements using QML properties
        // so I needed to define a signal and use a JS trick to do it...
        // Thanks to Nicolai! (http://forum.meego.com/showpost.php?p=28504&postcount=2)
        Component.onCompleted: {
            if (item1 != "")
                menuModel.append( {myName: menuLabel + " - " + item1, myLabel: item1, myParentLabel: menuLabel} )
            if (item2 != "")
                menuModel.append( {myName: menuLabel + " - " + item2, myLabel: item2, myParentLabel: menuLabel} )
            if (item3 != "")
                menuModel.append( {myName: menuLabel + " - " + item3, myLabel: item3, myParentLabel: menuLabel} )
            if (item4 != "")
                menuModel.append( {myName: menuLabel + " - " + item4, myLabel: item4, myParentLabel: menuLabel} )
        }

    }

    Component {
        id: menuDelegate
        Menuitem {
            state: "open"
            name: myName
            label: myLabel
            parentLabel: myParentLabel
            width: menu.width
            hPosition: menu.hPosition
            vPosition: menu.vPosition
            onSendmessage: {
                toggleState()
                mainpage.dispatchMessage(message)
            }
        }
    }




    Column {
        id: menuColumn
        visible: false
        spacing: globalSpacing
        y: (vPosition == "Bottom") ? globalSpacing : headerItem.height //+ globalSpacing
        x: (hPosition == "Right") ? width : -width

        // Menu items
        Repeater {
            id: menuRepeater
            model: menuModel
            delegate: menuDelegate }

        // Bottom spacer
        Rectangle {
            color: backgroundColor
            height: globalRadius
            width: menu.width }
    }



    // Defining the frame around the menu
    Rectangle { id: menuFrameTop
                visible: (vPosition != "Top")
                color: borderColor
                height: 1; width: menu.width - globalRadius
                y: 0; x: globalRadius }

    Rectangle { id: menuFrameBottom
                visible: (vPosition != "Bottom")
                color: borderColor
                height: 1; width: menu.width - globalRadius
                y: menu.height - 1; x: globalRadius }

    Rectangle { id: menuFrameRight
                visible: (hPosition != "Right")
                color: borderColor
                width: 1; height: menu.height - globalRadius * 2 + 1
                x: menu.width + globalRadius - 1; y: globalRadius }

    Rectangle { id: menuFrameLeft
                visible: (hPosition != "Left")
                color: borderColor
                width: 1; height: menu.height - globalRadius * 2 + 1
                x: 0; y: globalRadius }

    // Top left corner, up, right
    Image { visible: (hPosition == "Left") && (vPosition != "Top")
            x: 0; y: - globalRadius + 1
            source: "qrc:/images/bl_corner.png" }

    // Top right corner, down, left
    Image { visible: (hPosition != "Right") && (vPosition != "Top")
            x: menu.width; y: 0
            source: "qrc:/images/tr_corner.png" }

    // Bottom right corner, up, right
    Image { visible: (hPosition != "Right") && (vPosition == "Bottom")
            x: menu.width + globalRadius - 1; y: menu.height - globalRadius + 1
            source: "qrc:/images/bl_corner.png" }

    // Top left corner, down, right
    Image { visible: (hPosition != "Left") && (vPosition != "Top")
            x: 0; y: 0
            source: "qrc:/images/tl_corner.png" }

    // Bottom left corner, up, left
    Image { visible: (hPosition != "Left") && (vPosition == "Bottom")
            x: -globalRadius + 1; y: menu.height - globalRadius + 1
            source: "qrc:/images/br_corner.png" }

    // Top right corner, up, left
    Image { visible: (hPosition == "Right") && (vPosition != "Top")
            x: menu.width - globalRadius + 1; y: - globalRadius + 1
            source: "qrc:/images/br_corner.png" }

    // Bottom right corner, down, left
    Image { visible: (hPosition == "Right") && (vPosition != "Bottom")
            x: menu.width - globalRadius + 1; y: menu.height - 1
            source: "qrc:/images/tr_corner.png" }

    // Bottom left corner, up, right
    Image { visible: (hPosition != "Left") && (vPosition != "Bottom")
            x: 0; y: menu.height - globalRadius
            source: "qrc:/images/bl_corner.png" }

    // Top left corner, down, left
    Image { visible: (hPosition != "Left") && (vPosition == "Top")
            x: - globalRadius + 1; y: 0
            source: "qrc:/images/tr_corner.png" }

    // Top right corner, down, right
    Image { visible: (hPosition != "Right") && (vPosition == "Top")
            x: menu.width + globalRadius - 1; y: 0
            source: "qrc:/images/tl_corner.png" }

    // Bottom right corner, up, left
    Image { visible: (hPosition != "Right") && (vPosition != "Bottom")
            x: menu.width; y: menu.height - globalRadius
            source: "qrc:/images/br_corner.png" }

    // Bottom left corner, down, right
    Image { visible: (hPosition == "Left") && (vPosition != "Bottom")
            x: 0; y: menu.height - 1
            source: "qrc:/images/tl_corner.png" }






    states: [
        State {
            name: "hidden"
            PropertyChanges { target: menu; visible: false }
        },
        State {
            name: "headerIconSmall"
            PropertyChanges { target: menuColumn; visible: false }
            PropertyChanges { target: headerItem; large: false }
            PropertyChanges { target: headerItem; menuFolded: true }
            PropertyChanges { target: menu; width: headerItem.width }
        },
        State {
            name: "headerIconLarge"
//            when: headerArea.containsMouse
            PropertyChanges { target: menuColumn; visible: false }
            PropertyChanges { target: headerItem; large: true }
            PropertyChanges { target: headerItem; menuFolded: true }
            PropertyChanges { target: menu; width: headerItem.width }
        },
        State {
            name: "headerIconLabelSmall"
            PropertyChanges { target: menuColumn; visible: false }
            PropertyChanges { target: headerItem; large: false }
            PropertyChanges { target: headerItem; menuFolded: false }
            PropertyChanges { target: menu; width: headerItem.width }
        },
        State {
            name: "headerIconLabelLarge"
            PropertyChanges { target: menuColumn; visible: false }
            PropertyChanges { target: headerItem; large: true }
            PropertyChanges { target: headerItem; menuFolded: false }
            PropertyChanges { target: menu; width: headerItem.width }
        },
        State {
            name: "open"
            PropertyChanges { target: menuColumn; visible: true }
            PropertyChanges { target: menuColumn; x: 0}
            PropertyChanges { target: headerItem; large: true }
            PropertyChanges { target: menu; width: Math.max(menuItemWidth, headerItem.width) }
            PropertyChanges { target: menu; height: headerItem.height + menuColumn.height }
            PropertyChanges { target: headerItem; menuFolded: false }
        }
    ]


    transitions: [
        Transition {
            from: "open"
            NumberAnimation { properties: "opacity, width, height, x, y, iconsize"; easing.type: Easing.OutBack; duration: 1000 }
        },
        Transition {
            to: "open"
            NumberAnimation { properties: "opacity, width, height, x, y, iconsize"; easing.type: Easing.OutBack; duration: 500 }
        }
    ]
}
