import QtQuick 1.0

Rectangle {
    id: dialog
    signal clicked
    signal sendIP(string ipstring)
    signal sendPort(string portstring)
    property int space: globalRadius + globalSpacing
    property int fontsize: largeFontSize
    property string ipstring
    property string portstring

    anchors.horizontalCenter: parent.horizontalCenter
    anchors.verticalCenter: parent.verticalCenter

    width: iplabel.width + ipinput.width + 2*radius + 2*space
    height: iplabel.height + portlabel.height + 2*radius + 3*space
    color: backgroundColor
    radius: globalRadius
    border.color: borderColor
    border.width: borderWidth

    state: "invisible"
    visible: opacity > 0

    Image {
        id: icon
        source: "qrc:/images/Settings - Network80.png"
        anchors.horizontalCenter: parent.horizontalCenter
        y: - icon.height
    }

    Text {
        id: iplabel
        x: dialog.radius +space
        y: dialog.radius + space
        color:  textColor
        text: "IP Adress: "
        font.pointSize: dialog.fontsize
    }

    TextInput {
        id: ipinput
        x: iplabel.x + iplabel.width
        y: iplabel.y
        color:  textColor
        text: ipstring
        inputMask: "000.000.000.000;_"
        font.pointSize: dialog.fontsize
        Keys.onReturnPressed: portinput.focus = true
        Keys.onTabPressed: portinput.focus = true
    }

    Text {
        id: portlabel
        x: iplabel.x
        y: iplabel.y + iplabel.height + space
        color:  textColor
        text: "Port: "
        font.pointSize: dialog.fontsize
    }

    TextInput {
        id: portinput
        x: portlabel.x + portlabel.width
        y: portlabel.y
        color:  textColor
        text: portstring
        inputMask: "00000;_"
        font.pointSize: dialog.fontsize
        Keys.onReturnPressed: connectlabel.focus = true
        Keys.onTabPressed: connectlabel.focus = true
        IntValidator { id: valint; bottom: 1; top: 65535 }
        validator: valint
    }


    CuteButton {
        id: connectlabel
        anchors.right: dialog.right
        anchors.rightMargin: dialog.radius + space
        y: portlabel.y
        label: "Connect"
        onClicked: {
            dialog.ipstring = ipinput.text
            dialog.portstring = portinput.text
            dialog.sendIP(ipinput.text)
            dialog.sendPort(portinput.text)
            dialog.state = "invisible"
            dialog.clicked()
        }
    }

    states: [
        State {
            name: "invisible"
            PropertyChanges { target: dialog; opacity: 0 }
            PropertyChanges { target: parent; focus: true }
        },

        State {
            name: "visible"
            PropertyChanges { target: dialog; opacity: 1.0 }
        }
    ]

    transitions: Transition {
        NumberAnimation { properties: "opacity"; duration: 200 }
    }

    function toggle() {
        if (state == "visible")
            state = "invisible";
        else
            state = "visible";
    }

}
