import QtQuick 1.0

Rectangle {
    id: mainpage
    objectName: "mainpage"

    // Signals
    signal send(string msg)
    signal sendIP(string ip)
    signal sendPort(string port)
    signal quit()
    signal showNormal()

    //properties used throughout the program
    property int globalRadius: 10
    property int globalSpacing: 4
    property int borderWidth: 1
    property int largeiconsize: 80
    property int smalliconsize: 32
    property int smallFontSize: 12
    property int largeFontSize: 16
    property int headerHeight: largeiconsize + 2*globalRadius
    property int menuItemHeight: smalliconsize //+ 2*radius
    property int menuItemWidth: 200
    property string backgroundColor: "black"
    property string borderColor: "white"
    property string textColor: "white"
    property string hoveredTextColor: "Lime"
    property string selectedItemColor: "#7F7F7F"
    property string hoveredItemColor: "#1F1F1F"

    property string currentPatientId
    property string currentPatientTitle
    property string currentPatientFirstName
    property string currentPatientLastName

    property string currentDoctorId
    property string currentDoctorTitle
    property string currentDoctorFirstName
    property string currentDoctorLastName

    property string ipaddress
    property string port
    property bool rightHanded: true
    property bool tactileUi: false

    // Values need to be initialized, even when working in fullscreen mode
    width: 640; height: 480
    color:  backgroundColor

    function dispatchMessage(msg) {
        console.debug(msg)
        if      (msg == "Settings - Network")    { state = "ipDialog" }
        else if (msg == "Patient - Select")      { state = "patientSelectDialog"; send(msg) }
        else if (msg == "Patient - Scanner")     { state = "backgndImage"; send(msg) }
        else if (msg == "Doctor - Login")        { state = "doctorSelectDialog"; send(msg) }
        else if (msg == "Doctor - Logout")       { state = "noDialog"; doctorLogout() }
        else if (msg == "Doctor - Send message") { send(msg) }
        else if (msg == "Settings - Quit")       { state = "noDialog"; quit() }
    }

    function addPatient(idNumber, title, firstName, lastName) {
        patientList.addItem(idNumber, title, firstName, lastName)
    }

    function addDoctor(idNumber, title, firstName, lastName) {
        doctorList.addItem(idNumber, title, firstName, lastName)
    }

    function clearPatients() {
        patientList.clearAll()
    }

    function clearDoctors() {
        doctorList.clearAll()
    }

    function doctorLogout() {
        // To "logout" the doctor, we just delete its information (Basic! Some security should take place here)
        currentDoctorId= "";
        currentDoctorTitle = "";
        currentDoctorFirstName  = "";
        currentDoctorLastName = "";
    }

    function selectPatient() {
        currentPatientId        = patientList.selectedId;
        currentPatientTitle     = patientList.selectedTitle;
        currentPatientFirstName = patientList.selectedFirstName;
        currentPatientLastName  = patientList.selectedLastName;
    }

    function selectDoctor() {
        currentDoctorId        = doctorList.selectedId;
        currentDoctorTitle     = doctorList.selectedTitle;
        currentDoctorFirstName = doctorList.selectedFirstName;
        currentDoctorLastName  = doctorList.selectedLastName;
    }

    Rectangle {
        id: innerframe
        x: globalSpacing; y: x
        width: parent.width - 2*x
        height: parent.height - 2*y
        color: backgroundColor
        border.color: borderColor
        border.width: borderWidth
        radius: globalRadius
    }

    Image {
        id: bgdImage
        visible: false
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        asynchronous: true
        Connections{
            target:client
            onImageAvailable:{
                bgdImage.source = "image://imageprovider/drcute"
            }
        }
    }

    IPadress {
        id: ip
        state : "invisible"
        ipstring: ipaddress
        portstring: port
        onClicked: {
            mainpage.sendIP(ip.ipstring)
            mainpage.sendPort(ip.portstring)
            mainpage.state = "noDialog"
        }
    }

    // Patient list
    CuteList {
        id: patientList
        state : "invisible"
        title: "Select a patient from the list"
        iconName: "Patient"
        width:400
        height: 300
        onClose: {
            mainpage.state = "noDialog"
        }
        onSelect: {
            mainpage.state = "noDialog"
            selectPatient()
        }
    }

    // Doctor list
    CuteList {
        id: doctorList
        state : "invisible"
        title: "Select a doctor from the list"
        iconName: "Doctor"
        width:400
        height: 300
        onClose: {
            mainpage.state = "noDialog"
        }
        onSelect: {
            mainpage.state = "noDialog"
            selectDoctor()
        }
    }

    CuteMenu {
        id: menuDoctor
        menuLabel: "Doctor"
        header:  currentDoctorTitle + " " + currentDoctorLastName
        header2: currentDoctorFirstName
        header3: currentDoctorId
        item1: "Login"
        item2: "Logout"
        item3: "Send message"
        item4: ""
        hPosition: "Left"
        vPosition: "Bottom"
    }

    CuteMenu {
        id: menuPatients
        menuLabel: "Patient"
        header:  currentPatientTitle + " " + currentPatientLastName
        header2: currentPatientFirstName
        header3: currentPatientId
        item1: "Select"
        item2: "New"
        item3: "Scanner"
        item4: ""
        hPosition: "Right"
        vPosition: "Bottom"
    }

    CuteMenu {
        id: menuSettings
        menuLabel: "Settings"
        header: "Settings"
        header2: "IP: " + ipaddress
        header3: "Port: " + port
        item1: "Network"
        item2: "Help"
        item3: "Quit"
        item4: ""
        hPosition: "Left"
        vPosition: "Top"
    }

    states: [
        State {
            name: "noDialog"
            PropertyChanges { target: ip;          state: "invisible" }
            PropertyChanges { target: patientList; state: "invisible" }
            PropertyChanges { target: doctorList;  state: "invisible" }
            PropertyChanges { target: bgdImage;    visible: false }
        },
        State {
            name: "ipDialog"
            PropertyChanges { target: ip;          state: "visible" }
            PropertyChanges { target: patientList; state: "invisible" }
            PropertyChanges { target: doctorList;  state: "invisible" }
            PropertyChanges { target: bgdImage;    visible: false }
            PropertyChanges { target: menuDoctor;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuPatients;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuSettings;  state: "headerIconLabelSmall" }
        },
        State {
            name: "patientSelectDialog"
            PropertyChanges { target: ip;          state: "invisible" }
            PropertyChanges { target: patientList; state: "visible" }
            PropertyChanges { target: doctorList;  state: "invisible" }
            PropertyChanges { target: bgdImage;    visible: false }
            PropertyChanges { target: menuDoctor;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuPatients;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuSettings;  state: "headerIconLabelSmall" }
        },
        State {
            name: "doctorSelectDialog"
            PropertyChanges { target: ip;          state: "invisible" }
            PropertyChanges { target: patientList; state: "invisible" }
            PropertyChanges { target: doctorList;  state: "visible" }
            PropertyChanges { target: bgdImage;    visible: false }
            PropertyChanges { target: menuDoctor;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuPatients;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuSettings;  state: "headerIconLabelSmall" }
        },
        State {
            name: "backgndImage"
            PropertyChanges { target: ip;          state: "invisible" }
            PropertyChanges { target: patientList; state: "invisible" }
            PropertyChanges { target: doctorList;  state: "invisible" }
            PropertyChanges { target: bgdImage;    visible: true }
            PropertyChanges { target: menuDoctor;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuPatients;  state: "headerIconLabelSmall" }
            PropertyChanges { target: menuSettings;  state: "headerIconLabelSmall" }
        }
    ]
}
