#include "servernode.h"

ServerNode::ServerNode(QObject *parent, QTcpSocket *socket, Sql *db) :
    QObject(parent), tcpSocket(socket), sql(db)
{
}

void ServerNode::readString()
{
    // Create a QDataStream to read the message from the incoming connexion
    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_7);

    quint16 blockSize;
    qint64 bytes = tcpSocket->bytesAvailable();

    if (bytes < (int)sizeof(quint16))
        return;
    in >> blockSize;

    if (tcpSocket->bytesAvailable() < blockSize)
        return;

    QString newMessage;
    in >> newMessage;

    qDebug()<< "Received message: " << newMessage;

    QStringList msgList;
    msgList << "abc" << "def" << "ghi" << "jkl";
    sendStringList(msgList);
}

// Read a message received from a client
void ServerNode::readMessage()
{
    qDebug() << "Interpreting the received message";
    // Create a QDataStream to read the message from the incoming connexion
    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_7);

    quint16 blockSize;
    qint64 bytes = tcpSocket->bytesAvailable();

    // We try to read the size of the block
    if (bytes < (int)sizeof(quint16)) return;
    in >> blockSize;

    // if the amount of available bytes does not match what was announced, we exit
    if (tcpSocket->bytesAvailable() < blockSize) return;

    // We first read the 'Unique ID'
    QString uniqueId;
    in >> uniqueId;
    qDebug() << "Message received from: " << uniqueId;

    // Then we read the command
    QString cmd;
    in >> cmd;
    qDebug() << "Command: " << cmd;

    // Then the number of parameters
    QString countString;
    bool isSuccess;
    in >> countString;
    qDebug() << countString << "parameters";
    int count = countString.toInt(&isSuccess);

    QStringList msgList;
    if (isSuccess) {
        QString msg;
        for (int i=0; i<count; i++)
        {
            in >> msg;
            msgList << msg;
            qDebug() << msg;
        }
    }

    qDebug()<< "Received command: " << cmd << "from: " << uniqueId;
    interpretCommandAndAnswer(uniqueId, cmd, msgList);
}


void ServerNode::interpretCommandAndAnswer(const QString uniqueId, const QString &cmd, const QStringList &msgList)
{
    QStringList msgStringList;
    if (cmd == "Patient - Select")
    {
        msgStringList = sql->listPatients();
        msgStringList.insert(0, QString::number(msgStringList.count()));
        msgStringList.insert(0, cmd);
        msgStringList.insert(0, uniqueId);
    }
    else if (cmd == "Doctor - Login")
    {
        msgStringList = sql->listDoctors();
        msgStringList.insert(0, QString::number(msgStringList.count()));
        msgStringList.insert(0, cmd);
        msgStringList.insert(0, uniqueId);
    }
    else if (cmd == "Patient - Scanner")
    {
        msgStringList << uniqueId << "cmd" << "0";
    }

    if (cmd == "Patient - Scanner")
        sendDefaultImage();
    else
    {
        sendStringList(msgStringList);
        qDebug() << "Message sent to the client:" << msgStringList;
    }

}

void ServerNode::sendString(const QString &msg)
{
    QByteArray block;
    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_7);

    out << (quint16)0;
    out << msg;
    out.device()->seek(0);
    out << (quint16)(block.size() - sizeof(quint16));

    tcpSocket->write(block);
}

void ServerNode::sendStringList(const QStringList &msgList)
{
    qDebug() << "Sending StringList: " << msgList;

    QByteArray block;
    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_7);

    out << (quint16)0;
    QString msg;

    qDebug() << msgList.count();

    for (int i=0; i<msgList.count(); i++)
    {
        msg = msgList.at(i);
        out << msg;
        qDebug() << "Sending" << msg;
    }



    out.device()->seek(0);
    out << (quint16)(block.size() - sizeof(quint16));

    tcpSocket->write(block);
}

void ServerNode::sendImage(const QImage &img)
{
    QByteArray block;              // Construct a QByteArray object
    QBuffer buffer(&block);        // Construct a QBuffer object using the QbyteArray
//    img.save(&buffer, "PNG");              // Save the QImage data into the QBuffer
    img.save(&buffer, "JPG");              // Save the QImage data into the QBuffer
    tcpSocket->write(block);          // Send the QBuffer (QbyteArray) over a socket
}

void ServerNode::sendDefaultImage()
{
    qDebug() << "Sending an image";
    QImage image;
    bool imageIsLoaded = image.load(":/data/radio.jpg", "JPG");
    qDebug() << "Image is loaded: " << imageIsLoaded;
    image.setText("name", "Image Title");
    sendImage(image);
}

