#include "sql.h"

Sql::Sql(QObject *parent) :
    QObject(parent)
{
    doctorCount = 0;
    patientCount = 0;
    qDebug() << "Initializing the database";

    // Creating the data directory if it doesn't exist
    QDir().mkdir("../data");

    // We'll use only one db in this program, hence we do not need to specify a connection name
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
    db.setHostName("0x4e84");
    db.setDatabaseName("../data/MrHyde.sqlite");
    db.setUserName("MrHyde");
    //db.setPassword("12345");

    dbIsOpen = db.open();
    if (dbIsOpen)
        qDebug() << "Database is open";
    else
        qDebug() << "Database is not open!";
}

void Sql::doQuery(QString select, QString from)
{
    if (!dbIsOpen) return;
    QString queryString = QString("SELECT %1 FROM %2").arg(select).arg(from);
    QSqlQuery query(queryString);
    qDebug() << "Query:" << queryString;

    while (query.next())
    {
        QString value = query.value(0).toString();
        qDebug() << "Result:" << value;
    }
}

void Sql::createStructure()
{
    if (!dbIsOpen) return;
    QString queryString;
    QSqlQuery query;

    queryString = QString("CREATE TABLE IF NOT EXISTS doctors (id VARCHAR(30) UNIQUE PRIMARY KEY, title VARCHAR(5), firstname VARCHAR(30), lastname VARCHAR(30))");
    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
      qDebug() << "Table 'doctors' created";

    queryString = QString("CREATE TABLE IF NOT EXISTS patients (id VARCHAR(30) UNIQUE PRIMARY KEY, title VARCHAR(5), firstname VARCHAR(30), lastname VARCHAR(30))");
    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
        qDebug() << "Table 'patients' created";
}

void Sql::addDoctor(QString employeeId, QString title, QString firstName, QString lastName)
{
    if (!dbIsOpen) return;
    QString queryString;
    QSqlQuery query;

    queryString = QString("INSERT INTO doctors (id, title, firstname, lastname) VALUES (%1, '%2', '%3', '%4')")
            .arg(employeeId).arg(title).arg(firstName).arg(lastName);
    doctorCount++;

    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
        qDebug() << QString("Added doctor: %1 %2 %3").arg(title).arg(firstName).arg(lastName);
}

void Sql::addPatient(QString socialSecurityId, QString title, QString firstName, QString lastName)
{
    if (!dbIsOpen) return;
    QString queryString;
    QSqlQuery query;

    queryString = QString("INSERT INTO patients (id, title, firstname, lastname) VALUES (%1, '%2', '%3', '%4')")
            .arg(socialSecurityId).arg(title).arg(firstName).arg(lastName);
    patientCount++;

    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
        qDebug() << QString("Added patient: %1 %2 %3").arg(title).arg(firstName).arg(lastName);
}

QStringList Sql::listDoctors()
{
    QStringList resultList;
    if (!dbIsOpen) return resultList;
    QString queryString;
    QSqlQuery query;
    doctorCount = 0;

    queryString = QString("SELECT * FROM doctors");

    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
    {
        QSqlRecord rec = query.record();

        int cols = rec.count();
        for( int c=0; c<cols; c++ )

        for( int r=0; query.next(); r++ ) {
            for( int c=0; c<cols; c++ ) {
                resultList << query.value(c).toString();
            }
            doctorCount++;
        }
    }
    return resultList;
}

QStringList Sql::listPatients()
{
    QStringList resultList;
    if (!dbIsOpen) return resultList;
    QString queryString;
    QSqlQuery query;
    patientCount = 0;

    queryString = QString("SELECT * FROM patients");

    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
    {
        QSqlRecord rec = query.record();

        int cols = rec.count();
        for( int c=0; c<cols; c++ )

        for( int r=0; query.next(); r++ ) {
            for( int c=0; c<cols; c++ ) {
                resultList << query.value(c).toString();
            }
            patientCount++;
        }
    }
    return resultList;
}

void Sql::deleteDoctor(QString employeeId)
{
    if (!dbIsOpen) return;
    QString queryString;
    QSqlQuery query;

    queryString = QString("DELETE FROM names WHERE id = %1")
            .arg(employeeId);
    doctorCount--;

    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
        qDebug() << QString("Removed patient: %1").arg(employeeId).toAscii();
}

void Sql::deletePatient(QString socialSecurityId)
{
    if (!dbIsOpen) return;
    QString queryString;
    QSqlQuery query;

    queryString = QString("DELETE FROM names WHERE id = %1")
            .arg(socialSecurityId);
    doctorCount--;

    query.prepare(queryString);
    if( !query.exec() )
      qDebug() << query.lastError();
    else
        qDebug() << QString("Removed patient: %1").arg(socialSecurityId).toAscii();
}
